@@ -96,7 +96,7 @@ void board_destroy(void)
 /**
  * Refresh the board image (border)
  */
-void board_refresh(void)
+static void board_refresh(void)
 {
 	assert(border != NULL);
 	put_image(border, X_OFFSET - 36, Y_OFFSET - 36);
@@ -115,6 +115,8 @@ void board_set_player(Player *player, int display_your_turn)
 	char *turn;
 	int len;
 	
+	board_refresh();
+
 	if (display_your_turn)
 		len = strlen(get_msg(M_YOUR_TURN)) + strlen(player->name) + 1;
 	else
@@ -135,8 +137,8 @@ void board_set_player(Player *player, int display_your_turn)
 	else
 		snprintf(turn, len, "%s%s", get_msg(M_ROUND), player->name);
 	
-	clear_text(25, G_MSG_X, G_MSG_Y);
-	draw_message(turn, G_MSG_X, G_MSG_Y, FALSE);
+	clear_text(-1, G_MSG_X, G_MSG_Y);
+	draw_message(turn, G_MSG_X, G_MSG_Y, -1, FALSE);
 	
 	free(turn);
 
