@@ -55,21 +55,18 @@ static void load_font(void)
 	assert(cursor_pic != NULL);	
 }
 
-#define CHARWIDTH 10
-#define LINEYOFFSET (+3)
-#define LINEHEIGHT 18
-
 /**
  * Draw a message on screen and update the screen
  *
  * @param[in] message	The text to draw
  * @param[in] x		The X coordinate (in pixels)
  * @param[in] y		The Y coordinate (in pixels)
+ * @param[in] width	Available width (-1 means no constraint)
  * @param[in] cursor	Whether to draw a cursor after the text
  */
-void draw_message(const char *message, int x, int y, int cursor)
+void draw_message(const char *message, int x, int y, int width, int cursor)
 {
-	draw_message_with_update(message, x, y, cursor, TRUE);
+	draw_message_with_update(message, x, y, width, cursor, TRUE);
 }
 
 /**
@@ -78,16 +75,21 @@ void draw_message(const char *message, int x, int y, int cursor)
  * @param[in] message	The text to draw
  * @param[in] x		The X coordinate (in pixels)
  * @param[in] y		The Y coordinate (in pixels)
+ * @param[in] width	Available width (-1 means no constraint)
  * @param[in] cursor	Whether to draw a cursor after the text
  * @param[in] update	Whether to update the screen after drawing
  */
-void draw_message_with_update(const char *message, int x, int y, int cursor,
-			      int update)
+void draw_message_with_update(const char *message, int x, int y, int width,
+			      int cursor, int update)
 {
 	int i = 0;
 	int cur_x = x;
 	static SDL_Rect *dst_rect = NULL;
 	static SDL_Rect *src_rect = NULL;
+	int writable;
+	int pix_len;
+	int first_char, last_char;
+
 	if (dst_rect == NULL)
 		dst_rect = malloc(sizeof(SDL_Rect));
 	if (src_rect == NULL)
@@ -102,7 +104,28 @@ void draw_message_with_update(const char *message, int x, int y, int cursor,
 		load_font();
 	}
 	
-	for (i = 0; i < strlen(message); i ++) {
+	if (width > -1)
+		writable = width / CHARWIDTH;
+	else
+		writable = strlen(message);
+
+	pix_len = strlen(message) * CHARWIDTH;
+
+	if (!cursor) {
+		/* display first chars */
+		first_char = 0;
+		last_char = min(writable, strlen(message));
+	} else {
+		/* one place for the cursor */
+		writable--;
+		/* display last chars */
+		last_char = strlen(message);
+		first_char = last_char - writable;
+		if (first_char < 0)
+			first_char = 0;
+	}
+
+	for (i = first_char; i < last_char; i ++) {
 		dst_rect->x = cur_x;
 		dst_rect->y = y + LINEYOFFSET;
 		
@@ -143,7 +166,7 @@ void clear_text(int length, int x, int y)
 	
 	dst_rect->x = x;
 	dst_rect->y = y + LINEYOFFSET;
-	dst_rect->w = CHARWIDTH * length;
+	dst_rect->w = length > -1 ? CHARWIDTH * length : XS - dst_rect->x;
 	dst_rect->h = LINEHEIGHT;
 
 	SDL_FillRect(screen, dst_rect, 0x00000000);
