@@ -20,6 +20,10 @@
 #ifndef __GAME_H__
 #define __GAME_H__
 
+#include <SDL.h>
+#include <SDL_thread.h>
+#include <SDL_mutex.h>
+
 typedef struct _Game Game;
 
 struct _Game
@@ -39,11 +43,9 @@ struct _Game
 	int killed;
 	int last_joined;
 
-#if (!defined __MINGW32__ && !defined MAEMO)
-	pthread_cond_t  cond_wait;
-	pthread_cond_t  cond_move;
-	pthread_cond_t  cond_quit;
-#endif
+	SDL_cond *cond_wait;
+	SDL_cond *cond_move;
+	SDL_cond *cond_quit;
 };
 
 void game_free(Game *);
