@@ -48,7 +48,7 @@
  *
  * @return the number of eaten pawns
  */
-int compute_eats(PawnColor color)
+static int compute_eats(PawnColor color)
 {	
 	int x = 0;
 	int y = 0;
@@ -122,7 +122,14 @@ static void set_winner(void)
 	char *winner = NULL;
 	int i;
 	int winnernum = -1;
-	
+	int total_pawns = 0;
+
+	for (i = 0; i < game_num_players(); i++)
+		total_pawns += pawn_count((PawnColor)i);
+
+	if (total_pawns == 0)
+		return;
+
 	for (i = 0; i < game_num_players(); i++) {
 		int cnt = pawn_count((PawnColor)i);
 		if (cnt >= 2 && winnernum == -1) {
@@ -150,11 +157,106 @@ static void set_winner(void)
 		strcat(winner, get_msg(M_WINS));
 	}
 	
-	clear_text(20/* max len, probably */, G_MSG_X, G_MSG_Y);
-	draw_message(winner, G_MSG_X, G_MSG_Y, FALSE);
+	clear_text(-1, G_MSG_X, G_MSG_Y);
+	draw_message(winner, G_MSG_X, G_MSG_Y, -1, FALSE);
 
 	free(winner);
-	SDL_Delay(1500);	
+	SDL_Delay(1500);
+}
+
+static void ask_replacements(Player *player)
+{
+	LList *pawns = pawn_get_replacement_pending_pawns(player->color);
+	LList *cur = pawns;
+
+	arrow_draw_all(player);
+	
+	if (pawns) {
+		Tile *start = NULL, *end = NULL;
+		Pawn *moved_pawn = NULL;
+
+		while (!start || !start->pawn) {
+			start = player_select_tile(player, pawns);
+			if (start && start->pawn &&
+			    !llist_find(pawns, start->pawn))
+				start = NULL;
+			if (pawn_count(player->color) == 0) {
+				/* must have quit! */
+				return;
+			}
+			if (game_ended()) return;
+		}
+#ifdef DEBUG
+		printf("got start tile (%d, %d)\n", start->pos_x, start->pos_y);
+#endif
+		while (!end || !llist_find(start->pawn->just_ate_on, end)) {
+			LList *allowed = llist_append(NULL, start->pawn);
+			tile_highlight(start, 128);
+			for (cur = start->pawn->just_ate_on; 
+			     cur ; cur = cur->next) {
+				tile_draw((Tile *)cur->data);
+			}
+			arrow_draw_all(player);
+
+			end = player_select_tile(player, allowed);
+			llist_free(allowed); 
+			allowed = NULL;
+			if (pawn_count(player->color) == 0) {
+				/* must have quit! */
+				return;
+			}
+			if (game_ended()) return;
+			tile_draw(start);
+			if (start == end || end->pawn != NULL) {
+				
+				goto again;
+			}
+		}
+#ifdef DEBUG
+		printf("got end tile (%d, %d)\n", end->pos_x, end->pos_y);
+#endif			
+		
+		assert(start && start->pawn && llist_find(pawns, start->pawn));
+		assert(end && !end->pawn);
+
+		moved_pawn = start->pawn;
+		pawn_move_to(moved_pawn, end);
+		
+		if (tile_get_next_in_row(start, end))
+			tile_draw(tile_get_next_in_row(start, end));
+
+		/* repaint */
+		cur = pawns;
+		while (cur) {
+			Pawn *pawn = (Pawn *)cur->data;
+			LList *to_tiles = pawn->just_ate_on;
+			while(to_tiles) {
+				Tile *to_tile = (Tile *)to_tiles->data;
+				Tile *third = tile_get_next_in_row(pawn_get_tile(pawn), to_tile);
+				
+				tile_draw(pawn_get_tile(pawn));
+				tile_draw(to_tile);
+				if (third)
+					tile_draw(third);
+				to_tiles = to_tiles->next;
+			}
+			cur = cur->next;
+		}
+		
+		llist_free(moved_pawn->just_ate_on);
+		moved_pawn->just_ate_on = NULL;
+		
+		pawn_remove_just_eaten(end);
+			
+		SDL_UpdateRect(screen, 0, 0, 0, 0);
+
+		cur = pawns;
+
+again:
+		llist_free(pawns);
+		ask_replacements(player);
+		return;
+	}
 }
 
 void play_game(void)
@@ -316,148 +418,14 @@ end_loop:
 	set_winner();
 }
 
-void ask_replacements(Player *player)
-{
-	LList *pawns = pawn_get_replacement_pending_pawns(player->color);
-	LList *cur = pawns;
-
-	arrow_draw_all(player);
-	
-	if (pawns) {
-		Tile *start = NULL, *end = NULL;
-		Pawn *moved_pawn = NULL;
-
-		while (!start || !start->pawn) {
-			start = player_select_tile(player, pawns);
-			if (start && start->pawn &&
-			    !llist_find(pawns, start->pawn))
-				start = NULL;
-			if (pawn_count(player->color) == 0) {
-				/* must have quit! */
-				return;
-			}
-			if (game_ended()) return;
-		}
-#ifdef DEBUG
-		printf("got start tile (%d, %d)\n", start->pos_x, start->pos_y);
-#endif
-		while (!end || !llist_find(start->pawn->just_ate_on, end)) {
-			LList *allowed = llist_append(NULL, start->pawn);
-			tile_highlight(start, 128);
-			for (cur = start->pawn->just_ate_on; 
-			     cur ; cur = cur->next) {
-				tile_draw((Tile *)cur->data);
-			}
-			arrow_draw_all(player);
-
-			end = player_select_tile(player, allowed);
-			llist_free(allowed); 
-			allowed = NULL;
-			if (pawn_count(player->color) == 0) {
-				/* must have quit! */
-				return;
-			}
-			if (game_ended()) return;
-			tile_draw(start);
-			if (start == end || end->pawn != NULL) {
-				
-				goto again;
-			}
-		}
-#ifdef DEBUG
-		printf("got end tile (%d, %d)\n", end->pos_x, end->pos_y);
-#endif			
-		
-		assert(start && start->pawn && llist_find(pawns, start->pawn));
-		assert(end && !end->pawn);
-
-		moved_pawn = start->pawn;
-		pawn_move_to(moved_pawn, end);
-		
-		if (tile_get_next_in_row(start, end))
-			tile_draw(tile_get_next_in_row(start, end));
-
-		/* repaint */
-		cur = pawns;
-		while (cur) {
-			Pawn *pawn = (Pawn *)cur->data;
-			LList *to_tiles = pawn->just_ate_on;
-			while(to_tiles) {
-				Tile *to_tile = (Tile *)to_tiles->data;
-				Tile *third = tile_get_next_in_row(pawn_get_tile(pawn), to_tile);
-				
-				tile_draw(pawn_get_tile(pawn));
-				tile_draw(to_tile);
-				if (third)
-					tile_draw(third);
-				to_tiles = to_tiles->next;
-			}
-			cur = cur->next;
-		}
-		
-		llist_free(moved_pawn->just_ate_on);
-		moved_pawn->just_ate_on = NULL;
-		
-		pawn_remove_just_eaten(end);
-			
-		SDL_UpdateRect(screen, 0, 0, 0, 0);
-
-		cur = pawns;
-
-again:
-		llist_free(pawns);
-		ask_replacements(player);
-		return;
-	}
-}
-
-void stop_game(void)
-{
-	int i, j;
-
-	if (!game_inited()) {
-		notify_quit();
-		return;
-	}
-	
-	game_suspend(TRUE);
-	
-	for (j = SDL_ALPHA_TRANSPARENT; j <= SDL_ALPHA_OPAQUE; j++) {
-		for (i = 0; i < game_num_players(); i++) {
-			LList *pawns = pawn_get_all((PawnColor)i);
-			LList *cur;
-			for (cur = pawns; cur; cur = cur->next) {
-				Pawn *pawn = (Pawn *)cur->data;
-				pawn_highlight(pawn, j);
-				if (j == SDL_ALPHA_OPAQUE)
-					pawn->eaten = TRUE;
-			}
-			llist_free(pawns);
-		}	
-	}
-	for (i = 0; i < game_num_players(); i++) {
-		Player *player = player_get((PawnColor)i, FALSE, 
-					(InputSystemMethod)NULL);
-		if (player->method != INPUT_NETWORK) {
-#ifdef DEBUG
-			printf("sending end game for %d\n", i);
-#endif
-			net_end_game(i);
-		}
-	}
-	
-	game_suspend(FALSE);
-
-	notify_quit();
-}
-
 void end_game(void)
 {
 	int i;
 
 	if (!game_inited())
 		return;
-	
+
+	board_freeze();
 	game_suspend(TRUE);
 	
 	for (i = 0; i < game_num_players(); i++) {
@@ -483,6 +451,7 @@ void end_game(void)
 	}
 	game_suspend(FALSE);
 	net_close();
+	board_thaw();
 }
 
 int can_move_on_center(void) {
