@@ -33,12 +33,11 @@
 #include "utils.h"
 #include "board.h"
 #include "logic.h"
-#include "keyboard.h"
 #include "options.h"
 #include "net.h"
 #include "sound.h"
 
-void run_tests()
+static void run_tests()
 {
 #ifdef DEBUG
 #endif
@@ -57,9 +56,6 @@ int main(int argc, char *argv[])
 		*last_dirsep = '\0';
 	}
 	
-	if (argc > 1 && argv[1] && *argv[1]) {
-		net_set_server(argv[1]);
-	}
 	SDL_Init(SDL_INIT_EVERYTHING|SDL_INIT_NOPARACHUTE);
 	SDL_EnableUNICODE(1);
 	if ( (screen=SDL_SetVideoMode(XS, YS, BPP, FLAGS)) == NULL ) {
@@ -68,6 +64,8 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
+	srand(time(NULL));
+
 	sound_init();
 	sound_load_sounds();
 
@@ -83,10 +81,6 @@ int main(int argc, char *argv[])
 
 	run_tests();
 
-	init_mutexes();
-
-	init_keyboard();
-
 	net_init();
 
 	while (!should_quit()) {
@@ -107,9 +101,9 @@ int main(int argc, char *argv[])
 		board_destroy();
 	}
 
-	stop_keyboard();
-
 	sound_stop();
 
+	net_stop();
+
 	exit(0);
 }
