@@ -33,6 +33,8 @@ typedef enum {
 	M_PLAYER_LEAVES,
 	M_LEAVES,
 	M_YOUR_TURN,
+	M_OFFICIAL_SERVER,
+	M_LOCAL_SERVER,
 	M_NUM_MSGS
 } MsgId;
 
