@@ -279,9 +279,9 @@ static void player_kill(Player *player)
 		strcpy(msg, player->name);
 		strcat(msg, get_msg(M_LEAVES));
 	}
-	clear_text(20/* max len, probably */, G_MSG_X, G_MSG_Y);
-	draw_message(msg, G_MSG_X, G_MSG_Y, FALSE);
-	SDL_Delay(1500);	
+	clear_text(-1, G_MSG_X, G_MSG_Y);
+	draw_message(msg, G_MSG_X, G_MSG_Y, -1, FALSE);
+	delay_with_event_poll(1500);	
 	clear_text(strlen(msg), G_MSG_X, G_MSG_Y);
 	free(msg);	
 }
