@@ -25,30 +25,37 @@
  * This file contains the network server code.
  */
 
+#include <errno.h>
+#include <unistd.h>
+#include <string.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h> 
+
+#ifndef __MINGW32__
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
-#include <pthread.h>
+#else
+#include <winsock2.h>
+#endif
+
 #include <sys/time.h>
 #include <time.h>
-#include <signal.h>
+
+#include <SDL.h>
+#include <SDL_thread.h>
+#include <SDL_mutex.h>
+
 #include "netops.h"
 #include "llist.h"
 #include "game.h"
+#include "player.h"
+#include "pawn.h"
 
 static FILE *logfp = NULL;
-/* redefinitions to avoid sdl dependancy */
-typedef enum {
-	INPUT_LOCAL,
-	INPUT_NETWORK,
-	INPUT_AI,
-	NUM_INPUT_SYSTEMS
-} InputSystemMethod;
 
 static char *input_system_names(InputSystemMethod i) {
 	char *name[3] = {
@@ -61,18 +68,16 @@ static char *input_system_names(InputSystemMethod i) {
 	return name[i];
 };
 
-typedef enum {
-	PAWN_ORANGE = 0,
-	PAWN_BLUE,
-	PAWN_RED,
-	PAWN_GREEN,
-	PAWN_NUM_COLORS
-} PawnColor;
-
 static unsigned int cur_id = 0;
-pthread_mutex_t gamelist_mutex;
+SDL_mutex *gamelist_mutex = NULL;
 static LList *gamelist = NULL;
 
+#ifndef __MINGW32__
+static int optOn = 1;
+#else
+static const char optOn = 1;
+#endif
+
 typedef enum {
 	CONN_NEW,
 	CONN_SETUP_NEW,
@@ -92,6 +97,7 @@ typedef enum {
 
 typedef struct _ConnData {
 	int fd;
+	SDL_Thread *thread;
 	char *addr;
 	ConnState state;
 
@@ -106,7 +112,6 @@ typedef struct _ConnData {
 static ConnData *conn_data_init(int fd, const char *addr)
 {
 	ConnData *cd = malloc(sizeof(ConnData));
-	int optOn = 1;
 
 	if (cd == NULL)
 		return NULL;
@@ -115,6 +120,7 @@ static ConnData *conn_data_init(int fd, const char *addr)
 		&optOn, sizeof(optOn));
 
 	cd->fd = fd;
+	cd->thread = NULL;
 	cd->addr = strdup(addr);
 	cd->state = CONN_NEW;
 	cd->game = NULL;
@@ -154,11 +160,13 @@ static void do_log(ConnData *cd, const char *format, ...)
 {
 	va_list args;
 	struct timeval tv;
-	struct tm tm;
 	time_t t;
-
+#ifndef __MINGW32__
+	struct tm tm;
+#endif
 	gettimeofday(&tv, NULL);
 	t = time(NULL);
+#ifndef __MINGW32__
 	localtime_r(&t, &tm);
 
         fprintf(logfp, "%02d/%02d/%4d %02d:%02d:%02d.%04d [%s(%d)] - ", 
@@ -169,8 +177,14 @@ static void do_log(ConnData *cd, const char *format, ...)
 		    tm.tm_min,
 		    tm.tm_sec,
 		    (int)(tv.tv_usec / 1000),
-		    cd->addr,
-		    cd->fd);
+		    cd ? cd->addr : "Server ",
+		    cd ? cd->fd : 0);
+#else
+        fprintf(logfp, "%s [%s(%d)] - ", 
+		    asctime(localtime(&t)),
+		    cd ? cd->addr : "Server ",
+		    cd ? cd->fd : 0);
+#endif
 
         va_start(args, format);
         vfprintf(logfp, format, args);
@@ -190,9 +204,9 @@ static void free_game(Game *game)
 	for (i = 0; i < 4; i++)
 		free(game->player_name[i]);
 	
-	pthread_cond_destroy(&game->cond_wait);
-	pthread_cond_destroy(&game->cond_move);
-	pthread_cond_destroy(&game->cond_quit);
+	SDL_DestroyCond(game->cond_wait);
+	SDL_DestroyCond(game->cond_move);
+	SDL_DestroyCond(game->cond_quit);
 	free(game);
 }
 
@@ -249,6 +263,7 @@ static int get_nb_missing_players(Game *game)
 
 static void set_timeouts(ConnData *cd, int seconds)
 {
+#ifndef __MINGW32__
 	struct timeval timeo;
 	timeo.tv_sec = seconds;
 	timeo.tv_usec = 0;
@@ -256,9 +271,16 @@ static void set_timeouts(ConnData *cd, int seconds)
 		perror("setsockopt");
 	if (setsockopt(cd->fd, SOL_SOCKET, SO_SNDTIMEO, &timeo, sizeof(timeo)) < 0)
 		perror("setsockopt");
+#else
+	DWORD timeo = seconds * 1000;
+	if (setsockopt(cd->fd, SOL_SOCKET, SO_RCVTIMEO, (const char *)&timeo, sizeof(timeo)) < 0)
+		perror("setsockopt");
+	if (setsockopt(cd->fd, SOL_SOCKET, SO_SNDTIMEO, (const char *)&timeo, sizeof(timeo)) < 0)
+		perror("setsockopt");
+#endif
 }
 
-int server_setup_game(ConnData *cd, Game **new_game)
+static int server_setup_game(ConnData *cd, Game **new_game)
 {
 	char *p_name;
 	int gameid = 1;
@@ -367,13 +389,13 @@ int server_setup_game(ConnData *cd, Game **new_game)
 	do_log(cd, "   < game %s (%d) fully set up\n", (*new_game)->name,
 		(*new_game)->id);
 
-	pthread_cond_init(&(*new_game)->cond_wait, NULL);
-	pthread_cond_init(&(*new_game)->cond_move, NULL);
-	pthread_cond_init(&(*new_game)->cond_quit, NULL);
+	(*new_game)->cond_wait = SDL_CreateCond();
+	(*new_game)->cond_move = SDL_CreateCond();
+	(*new_game)->cond_quit = SDL_CreateCond();
 
-	pthread_mutex_lock (&gamelist_mutex);
+	SDL_LockMutex (gamelist_mutex);
 	gamelist = llist_append(gamelist, (*new_game));
-	pthread_mutex_unlock (&gamelist_mutex);
+	SDL_UnlockMutex (gamelist_mutex);
 	return 0;
 error:
 	free_game(*new_game);
@@ -405,7 +427,7 @@ static int server_send_games(ConnData *cd, int nump)
 	LList *avail = NULL, *cur;
 	int num, err;
 
-	pthread_mutex_lock (&gamelist_mutex);
+	SDL_LockMutex (gamelist_mutex);
 
 	avail = get_games(nump, 6);
 
@@ -441,7 +463,7 @@ static int server_send_games(ConnData *cd, int nump)
 	}
 	err = 0;
 error:
-	pthread_mutex_unlock (&gamelist_mutex);
+	SDL_UnlockMutex (gamelist_mutex);
 	llist_free(avail);
 	return err;
 }
@@ -527,7 +549,7 @@ error:
 	return game;
 }
 
-int server_get_game_id(ConnData *cd)
+static int server_get_game_id(ConnData *cd)
 {
 	unsigned char len;
 	int game_id;
@@ -751,11 +773,11 @@ static ConnState server_handle_conn_setup_preljoin(ConnData *cd)
 
 	do_log(cd, " > pre-reserving game ID %d\n", game_id);
 
-	pthread_mutex_lock (&gamelist_mutex);
+	SDL_LockMutex (gamelist_mutex);
 	game = server_do_reserve_game(cd, game_id);
 	if (game)
 		cd->previously_reserved = game->id;
-	pthread_mutex_unlock(&gamelist_mutex);
+	SDL_UnlockMutex(gamelist_mutex);
 
 	/* Reservation should have worked */
 	if (cd->previously_reserved < 0)
@@ -772,12 +794,12 @@ static ConnState server_handle_conn_setup_join(ConnData *cd)
 	/* player joins reserved game. */
 	int mynump;
 
-	pthread_mutex_lock (&gamelist_mutex);
+	SDL_LockMutex (gamelist_mutex);
 	cd->game = get_game(cd->previously_reserved);
 
 	if (cd->game == NULL) {
 		do_log(cd, " error: no pre-reserved game.\n");
-		pthread_mutex_unlock (&gamelist_mutex);
+		SDL_UnlockMutex (gamelist_mutex);
 		return CONN_KILL;
 	}
 	cd->previously_reserved = -1;
@@ -790,7 +812,7 @@ static ConnState server_handle_conn_setup_join(ConnData *cd)
 	do_log(cd, "  > player number %d\n", mynump);
 
 	if (mynump < 0) {
-		pthread_mutex_unlock(&gamelist_mutex);
+		SDL_UnlockMutex(gamelist_mutex);
 		return CONN_KILL;
 	}
 
@@ -802,7 +824,7 @@ static ConnState server_handle_conn_setup_join(ConnData *cd)
 			: "NULL");
 
 	if (cd->game->player_name[mynump] == NULL) {
-		pthread_mutex_unlock(&gamelist_mutex);
+		SDL_UnlockMutex(gamelist_mutex);
 		return CONN_KILL;
 	}
 
@@ -810,7 +832,7 @@ static ConnState server_handle_conn_setup_join(ConnData *cd)
 	cd->game->num_remote = cd->game->num_players - 1;
 	cd->game->last_joined = mynump;
 
-	pthread_mutex_unlock (&gamelist_mutex);
+	SDL_UnlockMutex (gamelist_mutex);
 	cd->player_num = mynump;
 
 	/* see if we can start game */
@@ -823,10 +845,10 @@ static ConnState server_handle_conn_wait_players(ConnData *cd)
 	 * be there. */
 	int nb_missing_players;
 
-	pthread_mutex_lock(&gamelist_mutex);
+	SDL_LockMutex(gamelist_mutex);
 
 	if (cd->game->killed) {
-		pthread_mutex_unlock(&gamelist_mutex);
+		SDL_UnlockMutex(gamelist_mutex);
 		return CONN_KILL;
 	}
 
@@ -836,18 +858,18 @@ static ConnState server_handle_conn_wait_players(ConnData *cd)
 		nb_missing_players, cd->game->id, cd->game->name);
 
 	if (server_wait_for_ping(cd) < 0) {
-		pthread_mutex_unlock(&gamelist_mutex);
+		SDL_UnlockMutex(gamelist_mutex);
 		return CONN_KILL;
 	}
 
 	if (server_send_waiting_for_players(cd, nb_missing_players) < 0) {
-		pthread_mutex_unlock(&gamelist_mutex);
+		SDL_UnlockMutex(gamelist_mutex);
 		return CONN_KILL;
 	}
 
 	if (server_send_last_join(cd, cd->game->last_joined,
 		cd->game->player_name[cd->game->last_joined]) < 0) {
-		pthread_mutex_unlock(&gamelist_mutex);
+		SDL_UnlockMutex(gamelist_mutex);
 		return CONN_KILL;
 	}
 
@@ -864,12 +886,12 @@ static ConnState server_handle_conn_wait_players(ConnData *cd)
 
 		cd->game->started = 1;
 
-		pthread_mutex_unlock(&gamelist_mutex);
+		SDL_UnlockMutex(gamelist_mutex);
 
 		return get_state_from_command(cd, "START_PLAY");
 	}
 
-	pthread_mutex_unlock(&gamelist_mutex);
+	SDL_UnlockMutex(gamelist_mutex);
 	sleep(1);
 
 	return get_state_from_command(cd, "WAIT_PLAYERS");
@@ -879,7 +901,7 @@ static ConnState server_handle_conn_start_play(ConnData *cd)
 {
 	int i;
 
-	pthread_mutex_lock(&gamelist_mutex);
+	SDL_LockMutex(gamelist_mutex);
 	cd->game->move_id = 0;
 	cd->game->cur_x = -1;
 	cd->game->cur_y = -1;
@@ -889,11 +911,11 @@ static ConnState server_handle_conn_start_play(ConnData *cd)
 	for (i = 0; i < cd->game->num_players; i++) {
 		if (send_msg(cd->fd, cd->game->player_name[i], 
 				strlen(cd->game->player_name[i])+1) < 0) {
-			pthread_mutex_unlock(&gamelist_mutex);
+			SDL_UnlockMutex(gamelist_mutex);
 			return CONN_KILL;
 		}
 	}
-	pthread_mutex_unlock(&gamelist_mutex);
+	SDL_UnlockMutex(gamelist_mutex);
 	return get_state_from_command(cd, "PLAY");
 }
 
@@ -919,14 +941,14 @@ static ConnState server_handle_conn_play_receive(ConnData *cd)
 
 	playercolor = ntohl(playercolor);
 
-	pthread_mutex_lock(&gamelist_mutex);
+	SDL_LockMutex(gamelist_mutex);
 
 	do_log(cd, " > game %d: player %d waiting for player %d move\n",
 		cd->game->id, cd->player_num, playercolor);
 
 
 	while (cd->game->move_id <= cd->move_id) {
-		pthread_cond_wait(&cd->game->cond_move, &gamelist_mutex);
+		SDL_CondWait(cd->game->cond_move, gamelist_mutex);
 		if (cd->game->killed)
 			break;
 	}
@@ -941,26 +963,25 @@ static ConnState server_handle_conn_play_receive(ConnData *cd)
 
 	if (send_msg(cd->fd, &x, 4) < 0) {
 		cd->game->waiting_acks--;
-		pthread_cond_broadcast(&cd->game->cond_wait);
-		pthread_mutex_unlock(&gamelist_mutex);
+		SDL_CondBroadcast(cd->game->cond_wait);
+		SDL_UnlockMutex(gamelist_mutex);
 		return CONN_KILL;
 	}
 	if (send_msg(cd->fd, &y, 4) < 0) {
 		cd->game->waiting_acks--;
-		pthread_cond_broadcast(&cd->game->cond_wait);
-		pthread_mutex_unlock(&gamelist_mutex);
+		SDL_CondBroadcast(cd->game->cond_wait);
+		SDL_UnlockMutex(gamelist_mutex);
 		return CONN_KILL;
 	}
 	do_log(cd, " < game %d: pushed move ID %d"
 		   " (x %d, y %d) from player %d to player %d\n",
 		cd->game->id, cd->game->move_id,
-		cd->game->cur_x, cd->game->cur_y,
-		playercolor, cd->player_num);
+		x, y, playercolor, cd->player_num);
 	cd->move_id++;
 
 	cd->game->waiting_acks--;
-	pthread_cond_broadcast(&cd->game->cond_wait);
-	pthread_mutex_unlock(&gamelist_mutex);
+	SDL_CondBroadcast(cd->game->cond_wait);
+	SDL_UnlockMutex(gamelist_mutex);
 
 	return CONN_PLAY;
 }
@@ -971,8 +992,8 @@ static void kill_game(Game *game)
 		return;
 
 	game->killed = 1;
-	pthread_cond_broadcast(&game->cond_wait);
-	pthread_cond_broadcast(&game->cond_move);
+	SDL_CondBroadcast(game->cond_wait);
+	SDL_CondBroadcast(game->cond_move);
 }
 
 static ConnState server_handle_conn_play_send(ConnData *cd, int quit)
@@ -980,35 +1001,35 @@ static ConnState server_handle_conn_play_send(ConnData *cd, int quit)
 	int playercolor, x, y;
 	unsigned char len = 0;
 
-	pthread_mutex_lock(&gamelist_mutex);
+	SDL_LockMutex(gamelist_mutex);
 	while (cd->game->waiting_acks > 0) {
 		/* wait for the previous move to be received by 
 		 * all */
-		pthread_cond_wait(&cd->game->cond_wait, &gamelist_mutex);
+		SDL_CondWait(cd->game->cond_wait, gamelist_mutex);
 		if (cd->game->killed) {
-			pthread_mutex_unlock(&gamelist_mutex);
+			SDL_UnlockMutex(gamelist_mutex);
 			return CONN_KILL;
 		}
 	}
 
 	if (read_msg(cd->fd, &playercolor, 4, &len) < 0) {
-		pthread_mutex_unlock(&gamelist_mutex);
+		SDL_UnlockMutex(gamelist_mutex);
 		return CONN_KILL;
 	}
 
 	playercolor = ntohl(playercolor);
 	if (playercolor != cd->player_num) {
-		pthread_mutex_unlock(&gamelist_mutex);
+		SDL_UnlockMutex(gamelist_mutex);
 		return CONN_KILL;
 	}
 
 	if (!quit) {
 		if (read_msg(cd->fd, &x, 4, &len) < 0) {
-			pthread_mutex_unlock(&gamelist_mutex);
+			SDL_UnlockMutex(gamelist_mutex);
 			return CONN_KILL;
 		}
 		if (read_msg(cd->fd, &y, 4, &len) < 0) {
-			pthread_mutex_unlock(&gamelist_mutex);
+			SDL_UnlockMutex(gamelist_mutex);
 			return CONN_KILL;
 		}
 
@@ -1019,33 +1040,38 @@ static ConnState server_handle_conn_play_send(ConnData *cd, int quit)
 		y = -2;
 	}
 
+	if (cd->game->killed) {
+		SDL_UnlockMutex(gamelist_mutex);
+		return CONN_KILL;
+	}
+
 	cd->game->cur_player = playercolor;
 	cd->game->cur_x = x;
 	cd->game->cur_y = y;
 	cd->game->move_id++;
 	cd->game->waiting_acks = cd->game->num_remote;
 
-	pthread_cond_broadcast(&cd->game->cond_move);
+	SDL_CondBroadcast(cd->game->cond_move);
 
 	do_log(cd, " < game %d: player %d sends move ID %d (x %d, y %d)\n",
 		cd->game->id, playercolor, cd->game->move_id, x, y);
 
 	if (quit) {
-		pthread_mutex_unlock(&gamelist_mutex);
+		SDL_UnlockMutex(gamelist_mutex);
 		return CONN_KILL;
 	}
 
 	/* wait here that everyone gets the move. 
 	 * this avoids deadlocks */
 	while (cd->game->waiting_acks > 0) {
-		pthread_cond_wait(&cd->game->cond_wait, &gamelist_mutex);
+		SDL_CondWait(cd->game->cond_wait, gamelist_mutex);
 		if (cd->game->killed) {
-			pthread_mutex_unlock(&gamelist_mutex);
+			SDL_UnlockMutex(gamelist_mutex);
 			return CONN_KILL;
 		}
 	}
 
-	pthread_mutex_unlock(&gamelist_mutex);
+	SDL_UnlockMutex(gamelist_mutex);
 	return CONN_PLAY;
 }
 
@@ -1053,7 +1079,7 @@ static ConnState server_handle_conn_kill(ConnData *cd)
 {
 	do_log(cd, "Connection done\n", cd);
 
-	pthread_mutex_lock(&gamelist_mutex);
+	SDL_LockMutex(gamelist_mutex);
 
 	if (cd->previously_reserved > 0)
 	{
@@ -1068,7 +1094,7 @@ static ConnState server_handle_conn_kill(ConnData *cd)
 
 	if (cd->game) {
 		cd->game->quit_players++;
-		pthread_cond_broadcast(&cd->game->cond_quit);
+		SDL_CondBroadcast(cd->game->cond_quit);
 
 		if (!cd->game->killed) {
 			kill_game(cd->game);
@@ -1097,7 +1123,7 @@ static ConnState server_handle_conn_kill(ConnData *cd)
 				- get_nb_missing_players(cd->game))
 			{
 				/* Wait for all others to be gone */
-				pthread_cond_wait(&cd->game->cond_quit, &gamelist_mutex);
+				SDL_CondWait(cd->game->cond_quit, gamelist_mutex);
 			}
 			do_log(cd, "finalizing game %d kill\n", cd->game->id);
 
@@ -1106,7 +1132,11 @@ static ConnState server_handle_conn_kill(ConnData *cd)
 			cd->game = NULL;
 		}
 	}
-	pthread_mutex_unlock(&gamelist_mutex);
+	SDL_UnlockMutex(gamelist_mutex);
+
+	do_log(cd, "Shutting down connection\n");
+	sock_close(cd->fd);
+
 	return CONN_EXIT;
 }
 
@@ -1143,7 +1173,7 @@ static ConnState server_handle_state(ConnData *cd)
 	}
 }
 
-static void *client_conn_handler(void *data)
+static int client_conn_handler(void *data)
 {
 	ConnData *cd = (ConnData *)data;
 
@@ -1151,71 +1181,214 @@ static void *client_conn_handler(void *data)
 
 	while (cd->state != CONN_EXIT)
 		cd->state = server_handle_state(cd);
-	
-	do_log(cd, "Shutting down connection\n");
-	shutdown(cd->fd, SHUT_RDWR);
-	close(cd->fd);
-	free(cd->addr);
-	free(cd);
 
-	pthread_exit(NULL);
+	return 0;
 }
 
-int main(int argc, char *argv[])
+static LList *clean_finished_threads(LList *threads_list)
 {
-	int portno = 8000;
-	int sockfd, clifd;
-	unsigned int clilen;
-	struct sockaddr_in serv_addr, cli_addr;
-	pthread_t threadid;
-	int optOn = 1;
+	LList *cur = threads_list;
 	
-	if (argc > 1) {
-		logfp = fopen(argv[1], "wb");
+	while (cur)  {
+		ConnData *cd = (ConnData *)cur->data;
+		if (cd->state == CONN_EXIT) {
+			int status;
+
+			SDL_WaitThread(cd->thread, &status);
+			do_log(cd, "Thread exited with status %d\n", status);
+			threads_list = llist_remove(threads_list, cd);
+			free(cd->addr);
+			free(cd);
+			cur = threads_list;
+		} else
+			cur = cur->next;
+	}
+	do_log(NULL, "%d threads running.\n", llist_length(threads_list));
+	return threads_list;
+}
+
+static int serversock = -1;
+SDL_mutex *serversock_mutex = NULL;
+
+static int timeout_sec = 5;
+static int do_broadcast = 0;
+static int broadcastsock = -1;
+static int magic_number = 0;
+
+void server_stop(void)
+{
+	SDL_LockMutex(serversock_mutex);
+	if (serversock < 0) {
+		SDL_UnlockMutex(serversock_mutex);
+		return;
+	}
+
+	do_log(NULL, "Stopping server.\n");
+
+	sock_close(serversock);
+	serversock = -1;
+
+	SDL_UnlockMutex(serversock_mutex);
+
+	if (broadcastsock >= 0) {
+		sock_close(broadcastsock);
+		broadcastsock = -1;
+	}
+}
+
+
+static int wait_for_connection(void)
+{
+	int sock;
+	
+	SDL_LockMutex(serversock_mutex);
+	sock = serversock;
+	SDL_UnlockMutex(serversock_mutex);
+
+	if (sock < 0)
+		return -1;
+
+	return sock_select(sock, timeout_sec * 1000);
+}
+
+void server_set_select_interval(int timeout)
+{
+	timeout_sec = timeout;
+}
+
+void server_set_do_broadcast(int enable, int magic)
+{
+	do_broadcast = enable;
+	magic_number = magic;
+}
+
+static void broadcast_setup(int sock)
+{
+	setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
+			&optOn, sizeof(optOn));
+	setsockopt(sock, SOL_SOCKET, SO_BROADCAST,
+			&optOn, sizeof(optOn));
+}
+
+static void broadcast_once(void)
+{
+	struct sockaddr_in br_addr;
+	int err;
+	char bcast[BCAST_LEN];
+
+	if (broadcastsock < 0) {
+		broadcastsock = socket(AF_INET, SOCK_DGRAM, 0);
+		broadcast_setup(broadcastsock);
+	}
+	if (broadcastsock < 0) {
+		do_log(NULL, "Can't broadcast: %s (%d)\n",
+			strerror(errno), errno);
+		return;
+	}
+
+	br_addr.sin_family = AF_INET;
+	br_addr.sin_port = htons(BCAST_PORT);
+	br_addr.sin_addr.s_addr = INADDR_BROADCAST;
+	
+	get_bcast_msg(bcast, BCAST_LEN, magic_number);
+
+	err = sendto(broadcastsock, bcast, sizeof(bcast),
+		0, (struct sockaddr *)&br_addr,
+		sizeof(br_addr));
+	if (err < 0) {
+#ifdef __MINGW32__
+		err = WSAGetLastError();
+#else
+		err = errno;
+#endif
+		do_log(NULL, "Can't send broadcast: %s (%d)\n",
+			strerror(err), err);
+	}
+}
+
+int server_start(const char *logfile)
+{
+	int clifd;
+#ifndef __MINGW32__
+	socklen_t clilen;
+#else
+	int clilen;
+#endif
+	struct sockaddr_in serv_addr, cli_addr;
+	LList *threads_list = NULL;
+	
+	if (logfile) {
+		logfp = fopen(logfile, "wb");
 		if (logfp == NULL) {
-			printf("can't open %s\n", argv[1]);
+			printf("can't open %s\n", logfile);
 			logfp = stdout;
 		}
 	} else
 		logfp = stdout;
 	
-	sockfd = socket(AF_INET, SOCK_STREAM, 0);
-	if (sockfd < 0) {
+	serversock = socket(AF_INET, SOCK_STREAM, 0);
+	if (serversock < 0) {
         	perror("Error on opening socket");
-		exit(1);
+		return -1;
 	}
+	
+	serversock_mutex = SDL_CreateMutex();
+	
 	memset((char *) &serv_addr, 0, sizeof(serv_addr));
 	serv_addr.sin_family = AF_INET;
 	serv_addr.sin_addr.s_addr = INADDR_ANY;
-	serv_addr.sin_port = htons(portno);
-	setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR,
+	serv_addr.sin_port = htons(NET_PORT);
+	setsockopt(serversock, SOL_SOCKET, SO_REUSEADDR,
 			&optOn, sizeof(optOn));
-	if (bind(sockfd, (struct sockaddr *) &serv_addr,
+	if (bind(serversock, (struct sockaddr *) &serv_addr,
 		sizeof(serv_addr)) < 0) {
 		perror("Error on binding");
-		exit(1);
+		return -1;
 	}
 	
-	listen(sockfd, 1000);
+	do_log(NULL, "Listening on %s\n", inet_ntoa(serv_addr.sin_addr));
+
+	listen(serversock, 1000);
 	
-	pthread_mutex_init(&gamelist_mutex, NULL);
+	gamelist_mutex = SDL_CreateMutex();
 	
 	clilen = sizeof(cli_addr);
-	while ((clifd = accept(sockfd,
-			      (struct sockaddr *)&cli_addr,
-			      &clilen)) >= 0) {
+	while (serversock >= 0) {
 
-		ConnData *cd = conn_data_init(clifd, inet_ntoa(cli_addr.sin_addr));
+		ConnData *cd;
 
-		if (cd != NULL) {
-			pthread_create(&threadid, NULL, client_conn_handler,
-				       (void *)cd);
-			pthread_detach(threadid);
+		if (do_broadcast)
+			broadcast_once();
+
+		if (wait_for_connection() < 0)
+			continue;
+
+		SDL_LockMutex(serversock_mutex);
+		if (serversock < 0) {
+			SDL_UnlockMutex(serversock_mutex);
+			break;
 		}
 
+		clifd = accept(serversock, (struct sockaddr *)&cli_addr, &clilen);
 		clilen = sizeof(cli_addr);	      
+		
+		SDL_UnlockMutex(serversock_mutex);
+
+		cd = conn_data_init(clifd, inet_ntoa(cli_addr.sin_addr));
+
+		if (cd != NULL) {
+			SDL_Thread *th = SDL_CreateThread(client_conn_handler, cd);
+			cd->thread = th;
+			threads_list = llist_append(threads_list, cd);
+		}
+
+		threads_list = clean_finished_threads(threads_list);
 	}
-	close(sockfd);
-	pthread_mutex_destroy(&gamelist_mutex);
+	SDL_DestroyMutex(gamelist_mutex);
+	SDL_DestroyMutex(serversock_mutex);
+	gamelist_mutex = NULL;
+	serversock_mutex = NULL;
+
+	do_log(NULL, "Exiting\n");
 	return 0;
 }
