@@ -37,7 +37,6 @@
 #include "utils.h"
 #include "board.h"
 #include "logic.h"
-#include "keyboard.h"
 #ifdef __MINGW32__
 #include <windows.h>
 #endif
@@ -92,26 +91,6 @@ void put_image(SDL_Surface *surface, int x, int y)
 		SDL_BlitSurface(surface, NULL, screen, tmp_rect);	
 }
 
-/**
- * Wait for key input
- *
- * @return the read key code
- */
-int wait_key(void)
-{
-	SDL_Event event;
-	int code = 0;
-	do {
-		SDL_WaitEvent(&event);
-		if (event.type == SDL_KEYUP) {
-			SDL_KeyboardEvent *kevent = &event.key;
-			code = kevent->keysym.sym;
-		}
-	} while (code == 0);
-	
-	return code;
-}
-
 static int num_players = 2;
 
 /**
@@ -245,64 +224,110 @@ int is_playing(void)
 	return playing;
 }
 
-static SDL_mutex *input_mutex = NULL;
+static int squit = FALSE;
 
 /**
- * Init Biloba's mutexes
+ * Signal that quitting has been requested
  */
-void init_mutexes(void)
+static void notify_quit(void)
 {
-	if (input_mutex == NULL)
-		input_mutex = SDL_CreateMutex();	
+	squit = TRUE;
 }
 
+int handle_generic_event(SDL_Event *event)
+{
+	int key = 0, mod = 0;
+	if (event->type == SDL_KEYDOWN) {
+		SDL_KeyboardEvent *kevent = &event->key;
+		if (kevent->keysym.sym && kevent->keysym.mod) {
+			key = kevent->keysym.sym;
+			if (key == SDLK_a)
+				key = SDLK_q; /* Ugly hack. */
+			mod = kevent->keysym.mod;
+		}
+	} else if (event->type == SDL_QUIT) {
+		key = SDLK_q;
+		mod = KMOD_CTRL;
+	}
+
+	if (key == SDLK_q && (mod & KMOD_CTRL)) {
+		notify_quit();
+		if (game_inited())
+			end_game();
+		return -2;
+	}
+	if (key == SDLK_ESCAPE && !game_inited()) {
+		notify_quit();
+		return -2;
+	}
+	if (key == SDLK_e && (mod & KMOD_CTRL) && game_inited()) {
+		end_game();
+		return -1;
+	}
+	if (key == SDLK_ESCAPE && game_inited()) {
+		end_game();
+		return -1;
+	}
+	return 0;
+}
+
+int event_poll(void)
+{
+	SDL_Event event;
+
+	while (SDL_PollEvent(&event)) {
+		int r = handle_generic_event(&event);
+		if (r < 0)
+			return r;
+	}
+	return 0;
+}
+
+#define INTERVAL_BETWEEN_CHECKS 200
+int delay_with_event_poll(int msec_timeout)
+{
+	int i, err;
+	int num_wait = msec_timeout / INTERVAL_BETWEEN_CHECKS;
+	int extra_wait = msec_timeout % INTERVAL_BETWEEN_CHECKS;
+	
+	for (i = 0; i < num_wait; i++) {
+		SDL_Delay(INTERVAL_BETWEEN_CHECKS);
+		err = event_poll();
+		if (err < 0)
+			return err;
+	}
+	if (extra_wait == 0)
+		return 0;
+
+	SDL_Delay(extra_wait);
+	return event_poll();
+}
+		
 /**
  * Wait for an SDL event
  *
  * @param[in] event_type	The event type to wait for
  *
  * @return the first event of event_type arrived
- * @attention SDL_USEREVENT and SDL_QUIT events are always
+ * @attention SDL_USEREVENT_QUIT and SDL_QUIT events are always
  * returned if we got one.
  */
 SDL_Event get_sdl_event(int event_type)
 {
 	SDL_Event event;
 	
-	SDL_LockMutex(input_mutex);
-	
 	do {
 		SDL_WaitEvent(&event);
-		if (event.type != event_type) {
-			if (event.type == SDL_KEYUP) {
-				SDL_KeyboardEvent *kevent = &event.key;
-				keyboard_push_event(kevent->keysym.sym, kevent->keysym.mod);
-			}
-			if (event.type == SDL_USEREVENT)
-				break; /* push it anyway */
-			if (event.type == SDL_QUIT) {
-				stop_game();
-				break;
-			}
-		}
-		if (game_ended())
+		if (event.type != event_type
+		    && handle_generic_event(&event) < 0) {
+			event.type = SDL_USEREVENT_QUIT;
 			break;
+		}
 	} while (event.type != event_type);
 
-	SDL_UnlockMutex(input_mutex);
-
 	return event;
 }
 
-static int squit = FALSE;
-
-/**
- * Signal that quitting has been requested
- */
-void notify_quit(void)
-{
-	squit = TRUE;
-}
 
 /**
  * Check whether quitting has been requested
