@@ -26,6 +26,9 @@
 
 //#define DEBUG 1
 
+#define SDL_USEREVENT_QUIT    (SDL_USEREVENT + 0)
+#define SDL_USEREVENT_REFRESH (SDL_USEREVENT + 1)
+
 #ifndef FALSE
 #define FALSE 0
 #endif
@@ -69,6 +72,10 @@
 #define DIR_SEP "\\"
 #endif
 
+#define PLAYER_NAME_LEN	32
+#define GAME_NAME_LEN 	32
+#define SERVER_LEN 	64
+
 extern char *progpath;
 extern char *langpath;
 
@@ -79,8 +86,6 @@ int get_y(int y);
 
 void put_image(SDL_Surface *surface, int x, int y);
 
-int wait_key(void);
-void notify_quit(void);
 int should_quit(void);
 void game_init(int bool);
 int game_inited(void);
@@ -101,12 +106,14 @@ int is_in_box(int x, int y, int a, int b, int c, int d);
 void set_playing(int playing);
 int is_playing(void);
 
-void init_mutexes(void);
 SDL_Event get_sdl_event(int event_type);
 
 SDL_Surface *biloba_load_image(const char *name);
 
 void set_language(void);
+int event_poll(void);
+int handle_generic_event(SDL_Event *event);
+int delay_with_event_poll(int msec_timeout);
 
 #ifdef MAEMO
 void SetWMName(const char *name, int is_fullscreen);
